//
//  MyCLController.m
//  SimpleRouteMeSample
//
// Copyright (c) 2009, Route-Me Contributors
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// * Redistributions of source code must retain the above copyright notice, this
//   list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

#import "MyCLController.h"

// Shorthand for getting localized strings, used in formats below for readability
#define LocStr(key) [[NSBundle mainBundle] localizedStringforKey: (key) \
														   value: @"" \
														   table: nil]

@implementation MyCLController

@synthesize delegate;
@synthesize locationManager;

- (id)init
{
	self = [super init];
	if (self != nil)
	{
		self.locationManager = [[[CLLocationManager alloc] init] autorelease];
		self.locationManager.delegate = self; // Tells the location manager to send updates to this object
	}
	
	return self;
}

// Called when the location is updated
- (void)locationManager: (CLLocationManager *)manager
	didUpdateToLocation: (CLLocation *)newLocation
		   fromLocation: (CLLocation *)oldLocation
{
	NSMutableString *update = [[[NSMutableString alloc] init] autorelease];
	
	// Timestamp
	NSDateFormatter *dateFormatter = [[[NSDateFormatter alloc] init] autorelease];
	[dateFormatter setDateStyle: NSDateFormatterMediumStyle];
	[dateFormatter setTimeStyle: NSDateFormatterMediumStyle];
	[update appendFormat: @"%@\n\n", [dateFormatter stringFromDate: newLocation.timestamp]];
	
	// Horizontal coordinates
	if (signbit(newLocation.horizontalAccuracy))
	{
		// Negative accuracy means an invalid or unavailable measurement
		[update appendString: LocStr(@"LatLongUnavailable")];
	}
	else
	{
		// CoreLocation returns positive for North & East, negative for South & West
		[update appendFormat: LocStr(@"LatLongFormat"), // This format takes 4 arge: 2 pairs of the form coordinate + compas direction
		 fabs(newLocation.coordinate.latitude), signbit(newLocation.coordinate.latitude) ? LocStr(@"South") : LocStr(@"North"),
		 fabs(newLocation.coordinate.longitude), signbit(newLocation.coordinate.longitude) ? LocStr(@"West") : LocStr(@"East")];
		[update appendString: @"\n"];
		[update appendFormat: LocStr(@"MeterAccuracyFormat"), newLocation.horizontalAccuracy];
	}
	
	[update appendString: @"\n\n"];
	
	// Altitude
	if (signbit(newLocation.verticalAccuracy))
	{
		// Negative accuracy means an invalid or unavailable measurement
		[update appendString: LocStr(@"AltUnavailable")];
	}
	else
	{
		// Positive and negative in altitude denote above & below sea level, respectively
		[update appendString: LocStr(@"AltitudeFormat"), fabs(newLocation.altitude), (signbit(newLocation.altitude)) ? LocStr(@"BelowSeaLevel") : LocStr(@"AboveSeaLevel")];
		[update appendString: @"\n"];
		[update appendFormat: LocStr(@"MeterAccuracyFormat"), newLocation.verticalAccuracy];
	}
	
	[update appendString: @"\n\n"];
	
	// Calculate distance moved and time elapsed, but only if we have an "old" location
	//
	// NOTE: Timestamps are based on when queries start, now when they return.
	// CoreLocation will query your location based on several methods. 
	// Sometimes, queries can come back in a different order from which they
	// were placed, which means the timestamp of the "old" location can sometimes
	// be newer than on the "new" location. For example, we will clamp the timeElapsed
	// to zero to avoid showing negative time in the ui
}

@end
